/*---------------------------------- Title ----------------------------------*/
/* File converter.                                                           */
/* Replaces each byte sequence 0D-0A (carriage_return - line_feed)           */
/*   by a single byte 0A (line_feed)                                         */
/* Programmer: Jens Groh                                                     */
/*---------------------------------------------------------------------------*/

/*------------------------- Preprocessor Directives -------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <alloc.h>

/*--------------------------------- Switches --------------------------------*/

/*----------------------------- Type Definitions ----------------------------*/

/*----------------------------- Global Constants ----------------------------*/
#define OF_NAME "out."

/*----------------------------- Global Variables ----------------------------*/
char *exitstr;
FILE *infile, *outfile;

/*---------------------------- Function Prototypes --------------------------*/
void exit_0(void);
void exit_1(void);
void exit_2(void);
unsigned int keycode(void);

/*----------------------------- Inline Functions ----------------------------*/

/*------------------------------- Main Program ------------------------------*/
int main(int argc,char **argv) {
   int i;
   int wrongparam = 0;
   unsigned char byte,prev_byte;
   int n = 0;
   long length, ip, op;
   char *infilename = NULL;
   char *outfilename = OF_NAME;

   atexit(exit_0);
   for (i = 1; i < argc; i++) {
      if (*argv[i] == '-') {
         switch (*(argv[i] + 1)) {
         case 'i':
         case 'I':
            infilename = argv[i] + 2;
            break;
         case 'o':
         case 'O':
            outfilename = argv[i] + 2;
            break;
         default:
            wrongparam = 1;
         }
      }
   }
   if ((infilename == NULL) || wrongparam) {
      exitstr = "\
CONVERT.EXE usage:\n\
convert -i<infile> [-o<outfile>]\n\
Parameters: (in any order, lower or upper case)\n\
  <infile>       : input file name\n\
  <outfile>      : output file name (default: `out.')";
      exit(EXIT_FAILURE);
   }
   if ((infile = fopen(infilename,"rb")) == NULL) {
      exitstr = "Cannot open input file.";
      exit(EXIT_FAILURE);
   }
   atexit(exit_1);
   if ((outfile = fopen(outfilename,"wb+")) == NULL) {
      exitstr = "Cannot open output file.";
      exit(EXIT_FAILURE);
   }
   atexit(exit_2);
   fseek(infile, 0L, SEEK_END);
   length = ftell(infile);
   fseek(infile, 0L, SEEK_SET);
   printf("Input File Length = %ld Bytes.\n", length);

   prev_byte = fgetc(infile);
   for (ip = 1L, op = 0L; ip < length; ip++) {
      if (++n >= 100) {
         n = 0;
         printf("Input: % 9ld  Output: % 9ld\r", ip, op);
      }
      byte = fgetc(infile);
      if (!(prev_byte == 0x0D && byte == 0x0A)) { /* On '0D-0A' skip '0D'. */
         fputc(prev_byte, outfile);
         op++;
      }
      prev_byte = byte;
   }
   fputc(prev_byte, outfile);

   printf("\n");
   exitstr = "";
   exit(EXIT_SUCCESS);
   return 0; /* never reached, just pro-forma */
}

/*--------------------------------- Function --------------------------------*/
void exit_0(void) {
   fprintf(stderr,"\n%s\n",exitstr);
}

/*--------------------------------- Function --------------------------------*/
void exit_1(void) {
   fclose(infile);
}

/*--------------------------------- Function --------------------------------*/
void exit_2(void) {
   fclose(outfile);
}

/*--------------------------------- Function --------------------------------*/
unsigned int keycode(void) {
   unsigned int c;
   c = getch(); /* read a character from keyboard */
   if(c == 0) /* did user hit a non-ASCII key? */
      c += getch() << 8; /* read scan code for keyboard */
   return c;
}

/*---------------------------------------------------------------------------*/

